<?php
/**
 * Created by JetBrains PhpStorm.
 * User: jorgefonseca
 * Date: 14/11/13
 * Time: 10:57
 * To change this template use File | Settings | File Templates.
 */
namespace Folha\Facade;

use Curriculo\Logic\Cadastros;
use Doctrine2\DBAL\Connection;
use Folha\Entities\GpeMovimentacao;
use \Folha\Logic\Cadastros\Vinculo\CadastroAutomaticoDependentes;
use \Folha\Logic\Cadastros\Vinculo\AlteracaoOcupacao;

class Vinculo {

    const WINDOW = 'gpe_vinculom_wnd_mixed';
    const WINDOW_RELACIONA_DEPENDENTE = "fpa_relacionaparc_por_vinculo";

    const TAB_SALARIO           = 'gpe_altesalario';
    const TAB_SITUACAOFUNCIONAL = 'gpe_altesitufunc';
    const TAB_MOVIMENTACAO      = 'gpe_movimentacao';
    const TAB_ESCALA            = 'gpe_alteescala';
    const TAB_OCUPACAO          = 'gpe_alteocupacao';

    const ATIVIDADE_NORMAL      = 1;

    /** @var $vinculo \GpevinculomEntity */
    static $vinculo;

    public static function gpeVinculohAfterInsert($nrorg, $nrvinculom) {

        $cadastrovinculo = new \Folha\Logic\Cadastros\Vinculo($nrorg);
        $cadastrovinculo->afterInsert($nrvinculom, $nrorg);

    }

    public static function gpeMovimentacaoAfterSave($nrorg, $nrvinculom) {

        $nrestruturam = tk_get_field_value('NRESTRUTURAM');
        $nrtipoestrutura = tk_get_field_value('NRTIPOESTRUTURA');
        $cadastromovimentacao = new \Folha\Logic\Cadastros\Vinculo\Movimentacao($nrorg);
        $cadastromovimentacao->afterSave($nrorg, $nrvinculom, $nrestruturam, $nrtipoestrutura);

    }

    /**
     * Limpa a referencia de estrutura no vinculo historico de acordo com o tipo de estrutura deletada.
     *
     * @param $nrorg
     * @param $nrvinculom
     *
     * leonardocarvalho - 22/08/2014
     * att gabrielcapanema - 12/05/2014
     * **agora utilizada no on delete**
     */
    public static function gpeMovimentacaoBeforeDelete($nrorg, $nrvinculom) {

        $nrtipoestrutura = tk_get_field_value('NRTIPOESTRUTURA');
        $cadastromovimentacao = new \Folha\Logic\Cadastros\Vinculo\Movimentacao($nrorg);
        $cadastromovimentacao->beforeDelete($nrorg, $nrvinculom, $nrtipoestrutura);

    }



    public static function gpeAltesitufuncAfterSave($nrorg, $nrvinculom) {

        $nrsitufuncm = tk_get_field_value('NRSITUFUNCM');
        $cadastroalteracaosituacao = new \Folha\Logic\Cadastros\Vinculo\AlteracaoSituacao($nrorg);
        $cadastroalteracaosituacao->afterSave($nrorg, $nrvinculom, $nrsitufuncm);

    }

    public static function gpeAlteocupacaoAfterSave($nrorg, $nrvinculom) {

        $nrocupacaom = tk_get_field_value('NROCUPACAOM');
        $cadastroalteracaoocupacao = new \Folha\Logic\Cadastros\Vinculo\AlteracaoOcupacao($nrorg);
        $cadastroalteracaoocupacao->afterSave($nrorg, $nrvinculom, $nrocupacaom);

    }

    public static function gpeAlteEscalaAfterSave($nrorg, $nrvinculom) {

        $nrescalatrabm = tk_get_field_value('NRESCALATRABM');
        $cadastroalteracaoescala = new \Folha\Logic\Cadastros\Vinculo\AlteracaoEscala($nrorg);
        $cadastroalteracaoescala->afterSave($nrorg, $nrvinculom, $nrescalatrabm);

    }

    public static function validaAbas() {
        self::validaSalario();
        self::validaSituacaoFuncional();
        self::validaMovimentacoes();
        self::validaEscala();
        self::validaOcupacao();
    }

    /**
     * @return \GpevinculomEntity
     */
    public static function getVinculo() {
        if (self::$vinculo) {
            return self::$vinculo;
        }
        $nrorg      = \FPA::getNrorgTrabalho();
        /** @var $vinculo \GpevinculomEntity */
        $nrvinculo = tk_get_env_var('GPEVINCULO_NRVINCULOM');

        if (!$nrvinculo || (tk_get_field_value('NRVINCULOM') && ($nrvinculo != tk_get_field_value('NRVINCULOM')))) {
            $nrvinculo  = tk_get_field_value('NRVINCULOM');
            tk_set_env_var('GPEVINCULO_NRVINCULOM', $nrvinculo);
        }
        self::$vinculo = \Gpevinculom::getObjectByWhere('NRORG = ? AND NRVINCULOM = ?', array($nrorg, $nrvinculo));
        return self::$vinculo;
    }

    public static function validaOcupacao() {
        $window = tk_get_active_window_name();
        if ($window != self::WINDOW) {
            return;
        }
        $eValido = true;
        /** @var $vinculo \GpevinculomEntity */
        $vinculo = self::getVinculo();
        $dtcompetencia = \FPA::getUltimoDiaMes();
        if (!is_object($vinculo) || !$vinculo->getOcupacao()) {
            $eValido = false;
        }
        tk_validate_tab(self::TAB_OCUPACAO, self::WINDOW, 'Vínculo sem ocupação', $eValido);
    }

    public static function validaEscala() {
        $window = tk_get_active_window_name();
        if ($window != self::WINDOW) {
            return;
        }
        $eValido = true;
        /** @var $vinculo \GpevinculomEntity */
        $vinculo = self::getVinculo();
        $dtcompetencia = \FPA::getUltimoDiaMes();
        if (!is_object($vinculo) || !$vinculo->getEscala()) {
            $eValido = false;
        }
        tk_validate_tab(self::TAB_ESCALA, self::WINDOW, 'Vínculo sem escala de trabalho', $eValido);
    }


    public static function validaSalario() {
        $window = tk_get_active_window_name();
        if ($window != self::WINDOW) {
            return;
        }
        $eValido = true;
        /** @var $vinculo \GpevinculomEntity */
        $vinculo = self::getVinculo();
        if (!is_object($vinculo) || !$vinculo->getAlteracoesSalario()) {
            $eValido = false;
        }
       tk_validate_tab(self::TAB_SALARIO, self::WINDOW, 'Vínculo sem alteração salarial', $eValido);
    }

    public static function validaSituacaoFuncional() {
        $window = tk_get_active_window_name();
        if ($window != self::WINDOW) {
            return;
        }
        $eValido = true;
        $mensagem = '';
        $dtcompetencia = \FPA::getUltimoDiaMes();

        /** @var $vinculo \GpevinculomEntity */
        $vinculo = self::getVinculo();
        if (is_object($vinculo)) {
            /** @var $alteracaoSituacao \GpealtesitufuncEntity */
            $alteracaoSituacao = $vinculo->getUltimaSituacaoFuncional($dtcompetencia);
            if (!is_object($alteracaoSituacao)) {
                $eValido = false;
                $mensagem = 'Vínculo sem situação funcional';
            } else if ($alteracaoSituacao->getNrsitufuncm() != 1 && $alteracaoSituacao->getNrsitufuncm() != 13) {
                $eValido = false;
                $mensagem = "A situação atual do vínculo é:
                {$alteracaoSituacao->getSituacaoFuncional()->getHistorico()->getNmsitufunch()}";
            }
        } else {
            $eValido = false;
            $mensagem = 'Vínculo sem situação funcional';
        }

       tk_validate_tab(self::TAB_SITUACAOFUNCIONAL, self::WINDOW, $mensagem, $eValido);
    }

    public static function validaMovimentacoes() {
        $window = tk_get_active_window_name();
        if ($window != self::WINDOW) {
            return;
        }
        $eValido = true;
        $dtcompetencia = \FPA::getUltimoDiaMes();
        $tipoEstruturaLegal = \FPA::getTipoEstruturaLegalOrg();

        /** @var $vinculo \GpevinculomEntity */
        $vinculo = self::getVinculo();
        if (!is_object($vinculo) || !$vinculo->getMovimentacoes($dtcompetencia)) {
            $eValido = false;
            $mensagem = 'Vínculo sem movimentações';
           tk_validate_tab(self::TAB_MOVIMENTACAO, self::WINDOW, $mensagem, $eValido);
            return;
        }

        $tiposEstrutura = array();
        /** @var $movimento \GpemovimentacaoEntity */
        foreach($vinculo->getMovimentacoes($dtcompetencia) as $movimento) {
            $tiposEstrutura[] = $movimento->getNrtipoestrutura();
        }
        if (!in_array(10, $tiposEstrutura) && $vinculo->getNrtipovinculom() == 1) {
            $mensagem .= 'Vínculo sem estrutura sindical<br>';
            $eValido = false;
        }
        if (!in_array($tipoEstruturaLegal, $tiposEstrutura)) {
            $mensagem .= 'Vínculo sem estrutura legal';
            $eValido = false;
        }

       tk_validate_tab(self::TAB_MOVIMENTACAO, self::WINDOW, $mensagem, $eValido);
    }

    /**
     * Retorna as chaves, nome e tipo de relacao dos dependentes de um vínculo.
     *
     *
     * @return String
     *
     * @author paulopereira - 20/11/2013
     */
    public static function retornaQueryRelacionaparcDeVinculo() 
    {
        $nrvinculom = tk_get_context_vars("NRVINCULOM", self::WINDOW);
        $nrorg      = tk_get_context_vars("NRORG", self::WINDOW);
        return \Parcnegocio::getParcnegocioDependenteDeVinculo($nrvinculom, $nrorg);
    }

    /**
     * Insere novos registros na tabela de fpa_depvinculo, de acordo com o tipo de relacionamento encontrado entre o 
     * vinculo e as diversas pessoas dependentes deste.
     *
     * @return void
     *
     * @author paulopereira - 22/11/2013
     */
    public static function executaCadastroAutomaticoDependentes() 
    {
         try {
            $dependentes = tk_get_checked_rows();
            $nrvinculom  = tk_get_context_vars("NRVINCULOM", self::WINDOW);
            $nrorg       = tk_get_context_vars("NRORG", self::WINDOW);

            $cadastroDependentes = new CadastroAutomaticoDependentes($nrvinculom, $nrorg);
            $cadastroDependentes->executaCadastro($dependentes);
            tk_close_window(self::WINDOW_RELACIONA_DEPENDENTE, true);
         } catch (\Exception $e) {
             tk_warning($e->getMessage());
         }

    }

    public static function getObjDependenteDeVinculo($nrorg, $nrvinculom, $tiporelaciona){
        $vinculo = \Folha\Entities\GpeVinculom::findByNrorgAndNrvinculom($nrorg, $nrvinculom);
        $pessoa = \Folha\Entities\GpePessoa::findByNrorgAndNrpessoa($nrorg, $vinculo->getNrpessoa());
        $dependentes = \Indeterminado\Entities\Relacionaparc::findByNrorgAndNrparcnegocioAndTiporelaciona($nrorg, $pessoa->getNrparcnegocio(), $tiporelaciona);
        return $dependentes;
    }


}